﻿namespace SelfPropelledArtilleryUnit.Entities;
/// <summary>
/// Класс-сущность "Бронированная машина"
/// </summary>
public class EntityPropelledArtillery
{
    /// <summary>
    /// Скорость
    /// </summary>
    public int Speed { get; private set; }
    /// <summary>
    /// Вес
    /// </summary>
    public double Weight { get; private set; }
    /// <summary>
    /// Основной цвет
    /// </summary>
    public Color BodyColor { get; private set; }
    public void setBodyColor(Color color)
    {
        BodyColor = color;
    }
    /// <summary>
    /// Шаг перемещения артеллерийской установки
    /// </summary>
    public double Step => Speed * 100 / Weight;
    /// <summary>
    /// Конструктор сущности 
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>
    public EntityPropelledArtillery(int speed, double weight, Color bodyColor)
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
    }

    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public virtual string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityPropelledArtillery), Speed.ToString(), Weight.ToString(), BodyColor.Name };
    }
    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityPropelledArtillery? CreateEntityPropelledArtillery(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(EntityPropelledArtillery))
        {
            return null;
        }
        return new EntityPropelledArtillery(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }
}
