﻿namespace SelfPropelledArtilleryUnit.Entities;
/// <summary>
/// Класс-сущность "Самоходная арт. установка"
/// </summary>
public class EntitySelfPropelledArtilleryUnit : EntityPropelledArtillery
{
    /// <summary>
    /// Дополнительный цвет (для опциональных элементов)
    /// </summary>
    public Color AdditionalColor { get; private set; }
    public void setAdditionalColor(Color color)
    {
        AdditionalColor = color;
    }
    /// <summary>
    /// Признак - Башня с орудием
    /// </summary>
    public bool TurretCannon { get; private set; }
    /// <summary>
    /// Признак - Залповая батарея сзади
    /// </summary>
    public bool LaunchBattery { get; private set; }
    /// <summary>
    /// Шаг перемещения артеллерийской установки
    /// </summary>
    public double Step => Speed * 100 / Weight;

    /// <summary>
    /// Инициализация полей объекта-класса EntitySelfPropelledArtilleryUnit
    /// </summary>
    /// <param name="additionalcolor">Дополнительный цвет</param>
    /// <param name="turretcannon">Башня с орудием</param>
    /// <param name="launchbattery">Залповая батарея сзади</param>
    public EntitySelfPropelledArtilleryUnit(int speed, double weight, Color bodycolor, Color additionalcolor, bool turretcannon, bool launchbattery) : base(speed, weight, bodycolor)
    {
        AdditionalColor = additionalcolor;
        TurretCannon = turretcannon;
        LaunchBattery = launchbattery;
    }
    /// <summary>
    /// Получение строк со значениями свойств продвинутого объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntitySelfPropelledArtilleryUnit), Speed.ToString(), Weight.ToString(), BodyColor.Name, AdditionalColor.Name,
                        TurretCannon.ToString(), LaunchBattery.ToString()};
    }
    /// <summary>
    /// Создание продвинутого объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntitySelfPropelledArtilleryUnit? CreateEntitySelfPropelledArtilleryUnit(string[] strs)
    {
        if (strs.Length != 7 || strs[0] != nameof(EntitySelfPropelledArtilleryUnit))
        {
            return null;
        }
        return new EntitySelfPropelledArtilleryUnit(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]),
                                    Color.FromName(strs[4]), Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]));
    }
}

